from grazie.api.client.llm_parameters import BaseParameters
from grazie.api.client.v8.parameters import Parameters


class LLMParameters(BaseParameters):
    # Speeds up and optimizes model responses when the response is supposed to be a minor modification of existing content.
    # Work only with OpenAI GPT 4.1, GPT-4o, and GPT-4o-mini series models.
    PredictedOutput = Parameters.StrKey("llm.parameters.predicted-output")

    # Lets you define how much time the models will spend thinking before they provide an answer.
    # Available for OpenAI reasoning models, such as o1, o3, and o4 series models.
    ReasoningEffort = Parameters.StrKey("llm.parameters.reasoning-effort")

    # Defines how many responses the model should generate for a single input message.
    NumberOfChoices = Parameters.IntKey("llm.parameters.number-of-choices")

    # Indicates the insertion location of cache points.
    # Available only for Anthropic models.
    CachePoints = Parameters.JsonKey("llm.parameters.cache-points")

    # JSON-format tool definition
    Tools = Parameters.JsonKey("llm.parameters.tools")

    # Run the tools in parallel. Enabled by default.
    ParallelToolCalls = Parameters.BooleanKey("llm.parameters.parallel-tool-calls")

    # The way how the model should handle tool calls.
    ToolChoiceAuto = Parameters.BooleanKey("llm.parameters.tool-choice-auto")
    ToolChoiceNone = Parameters.BooleanKey("llm.parameters.tool-choice-none")
    ToolChoiceRequired = Parameters.BooleanKey("llm.parameters.tool-choice-required")
    ToolChoiceNamed = Parameters.StrKey("llm.parameters.tool-choice-named")



from enum import Enum
from typing import Optional

import attrs

from grazie.api.client.chat.prompt import ChatPrompt
from grazie.api.client.quota.response import Quota, QuotaCredit
from grazie.api.client.utils import typ_or_none


@attrs.define(auto_attribs=True)
class LLMChatEventV5:
    type: str
    content: str
    name: Optional[str] = None


class LLMChatEventTypeV6(str, Enum):
    Content = "Content"
    FunctionCall = "FunctionCall"
    QuotaMetadata = "QuotaMetadata"


@attrs.define(auto_attribs=True)
class LLMChatEvent:
    type: LLMChatEventTypeV6 = attrs.field(
        converter=LLMChatEventTypeV6  # pyright: ignore [reportGeneralTypeIssues]
    )
    content: str = ""
    name: Optional[str] = None
    updated: Optional[Quota] = typ_or_none(Quota)
    spent: Optional[QuotaCredit] = typ_or_none(QuotaCredit)


@attrs.define(auto_attribs=True, frozen=True)
class ChatResponse:
    prompt: ChatPrompt
    content: str
    function_call: Optional[str] = None
    updated: Optional[Quota] = None
    spent: Optional[QuotaCredit] = None


@attrs.define(auto_attribs=True, frozen=True)
class ChatResponseStream:
    chunk: str
    function_call: Optional[str] = None
    updated: Optional[Quota] = None
    spent: Optional[QuotaCredit] = None



#!/usr/bin/env python3

import argparse
import os
from contextlib import nullcontext

from grazie.api.client._common import LLMChatApiVersion, LLMCompletionApiVersion
from grazie.api.client.chat.prompt import ChatPrompt as ChatPromptV6
from grazie.api.client.completion.prompt import CompletionPrompt as CompletionPromptV3
from grazie.api.client.endpoints import GrazieApiGatewayUrls
from grazie.api.client.gateway import AuthType, GrazieAgent, GrazieApiGatewayClient
from grazie.api.client.profiles import LLMProfile, Profile
from grazie.api.client.v8.chat.prompt import ChatPrompt as ChatPromptV8
from grazie.api.client.v8.completion.prompt import (
    CompletionPrompt as CompletionPromptV8,
)


def get_client(args: argparse.Namespace) -> GrazieApiGatewayClient:
    return GrazieApiGatewayClient(
        grazie_agent=GrazieAgent(name="grazie-api-gateway-client", version="dev"),
        url=args.gateway,
        auth_type=AuthType.USER,
        grazie_jwt_token=args.token,
    )


def completion(args: argparse.Namespace) -> None:
    client = get_client(args)

    if args.version == "3" or args.version is None:
        response = client.complete(
            prompt=CompletionPromptV3(args.prompt),
            profile=Profile.get_by_name(args.profile),
        )
        print(response.completion)

    elif args.version == "8":
        response = client.v8.complete(
            prompt=CompletionPromptV8(args.prompt),
            profile=Profile.get_by_name(args.profile),
        )
        print(response.content)

    else:
        print(f"Completion version {args.version} is not supported.")


def chat(args: argparse.Namespace) -> None:
    client = get_client(args)

    if args.version == "6" or args.version is None:
        response = client.chat(
            chat=ChatPromptV6().add_system(args.system).add_user(args.prompt),
            profile=Profile.get_by_name(args.profile),
        )
        print(response.content)

    elif args.version == "8":
        response = client.v8.chat(
            chat=ChatPromptV8().add_system(args.system).add_user(args.prompt),
            profile=Profile.get_by_name(args.profile),
        )
        print(response.content)

    else:
        print(f"Chat version {args.version} is not supported.")


def main() -> None:
    profiles = sorted(
        [
            value.name
            for attr in dir(Profile)
            for value in (getattr(Profile, attr),)
            if isinstance(value, LLMProfile)
        ]
    )

    parser = argparse.ArgumentParser(description="Grazie API Gateway Client")
    parser.add_argument(
        "-g", "--gateway", type=str, required=False, default=GrazieApiGatewayUrls.STAGING
    )
    parser.add_argument(
        "-p", "--profile", required=False, choices=profiles, default=Profile.OPENAI_CHAT_GPT.name
    )
    parser.add_argument(
        "-t",
        "--token",
        type=str,
        default=os.environ.get("GRAZIE_JWT_TOKEN", None),
        help=argparse.SUPPRESS,
    )
    subparsers = parser.add_subparsers()

    with nullcontext(subparsers.add_parser("completion")) as subparser:
        subparser.add_argument(
            "-v",
            "--version",
            required=False,
            choices=[str(x.value) for x in LLMCompletionApiVersion],
            default=None,
        )
        subparser.add_argument("prompt")
        subparser.set_defaults(func=completion)

    with nullcontext(subparsers.add_parser("chat")) as subparser:
        subparser.add_argument(
            "-v",
            "--version",
            required=False,
            choices=[str(x.value) for x in LLMChatApiVersion],
            default=None,
        )
        subparser.add_argument("-s", "--system", type=str, default="You are a helpful assistant.")
        subparser.add_argument("prompt")
        subparser.set_defaults(func=chat)

    args = parser.parse_args()

    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



from typing import Dict, List

import attr

from grazie.api.client._common import DeprecatedError
from grazie.api.client.v8.chat.message import (
    AssistantChatMessage,
    AssistantChatMessageTool,
    ChatMessage,
    ChatToolMessage,
    TextChatMessage,
)
from grazie.api.client.v8.chat.roles import ChatRole

FUNC_DEPRECATION_MSG = "Function calls are deprecated. Use add_tool instead."


@attr.s(auto_attribs=True, frozen=False)
class ChatPrompt:
    messages: List[ChatMessage] = attr.ib(default=attr.Factory(list))

    def _add_text(self, item: str, role: ChatRole):
        self.messages.append(TextChatMessage(role, item))
        return self

    def add_user(self, item: str):
        return self._add_text(item, ChatRole.USER)

    def add_system(self, item: str):
        return self._add_text(item, ChatRole.SYSTEM)

    def add_assistant(self, item: str):
        self.messages.append(AssistantChatMessage(text=item))
        return self

    def add_assistant_function(self, name: str, content: str):
        raise DeprecatedError(FUNC_DEPRECATION_MSG)

    def add_function(self, name: str, content: str):
        raise DeprecatedError(FUNC_DEPRECATION_MSG)

    def add_tool(self, id: str, tool_name: str, content: str, result: str):
        """
        Used to specify the result of the tool execution.

        :param id: The unique tool identifier. Can be an arbitrary string.
        :param tool_name: Name of the tool to which the message is related.
        :param content: The content of the response to the tool call.
        :param result: The result of running the tool with the provided parameters.
        """
        self.messages.append(AssistantChatMessageTool(id, tool_name, content))
        self.messages.append(ChatToolMessage(id, tool_name, result))
        return self

    def get_messages(self) -> List[Dict[str, str]]:
        return [message.to_json() for message in self.messages]



from enum import Enum
from typing import Optional


class LLMChatApiVersion(Enum):
    V6 = 6
    V8 = 8


class LLMCompletionApiVersion(Enum):
    V3 = 3
    V8 = 8


class StopReason(str, Enum):
    STOP = "stop"
    LENGTH = "length"
    FUNCTION_CALL = "function_call"
    TOOL_CALL = "tool_call"
    UNKNOWN = "unknown"


class UnknownMetadataResponse:
    """
    Simple container for any number of arbitrary fields without validation
    """

    content: Optional[str] = None

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def __repr__(self):
        fields = ", ".join(f"{k}={v!r}" for k, v in self.__dict__.items())
        return f"{self.__class__.__name__}({fields})"


class DeprecatedError(Exception):
    pass



import json
from typing import Any, Dict

import attrs

from grazie.api.client.parameters import BaseParameters
from grazie.api.client.v8.llm_tools import ToolDefinition


class Parameters(BaseParameters):
    @attrs.define(auto_attribs=True, frozen=True)
    class JsonValue(BaseParameters.Value):  # type: ignore[reportIncompatibleVariableOverride]
        value: Any

        def serialize(self) -> Dict[str, Any]:
            return {"type": "json", "value": json.dumps(self.value), "modified": 0}

        @classmethod
        def from_tools(cls, *tools: ToolDefinition):
            return cls(value=[attrs.asdict(tool) for tool in tools])

    _allowed_type_mapping = {
        (BaseParameters.BooleanKey, BaseParameters.BooleanValue),
        (BaseParameters.IntKey, BaseParameters.IntValue),
        (BaseParameters.FloatKey, BaseParameters.FloatValue),
        (BaseParameters.StrKey, BaseParameters.StrValue),
        (BaseParameters.JsonKey, JsonValue),
    }



from enum import Enum
from typing import List, Optional, Union

import attrs

from grazie.api.client._common import StopReason, UnknownMetadataResponse
from grazie.api.client.quota.response import QuotaResponse
from grazie.api.client.v8.chat.prompt import ChatPrompt


class LLMChatEventType(str, Enum):
    Content = "Content"
    QuotaMetadata = "QuotaMetadata"


@attrs.define(auto_attribs=True, frozen=True)
class ChatContentStreamChunk:
    content: str
    choiceIndex: Optional[int] = None


@attrs.define(auto_attribs=True, frozen=True)
class ChatToolCallStreamChunk:
    choiceIndex: Optional[int] = None
    parallelToolIndex: Optional[int] = None
    id: Optional[str] = None
    name: Optional[str] = None
    content: str = ""


@attrs.define(auto_attribs=True, frozen=True)
class ChatFinishMetadataResponse:
    reason: StopReason = StopReason.UNKNOWN
    choiceIndex: Optional[int] = None
    content: str = ""


@attrs.define(auto_attribs=True, frozen=True)
class ChatToolResponse:
    content: str
    parallelToolIndex: Optional[int] = None
    id: Optional[str] = None
    name: Optional[str] = None


@attrs.define(auto_attribs=True, frozen=True)
class ChatSingleResponse:
    stop_reason: Optional[str] = None
    content: Optional[str] = None
    tool_calls: List[ChatToolResponse] = []


@attrs.define(auto_attribs=True, frozen=True)
class ChatResponse:
    prompt: ChatPrompt
    responses: List[ChatSingleResponse]
    quota: QuotaResponse
    unknown_metadata: Optional[UnknownMetadataResponse] = None

    @property
    def content(self) -> str:
        """
        Aggregate texts from all responses.
        """
        texts: List[str] = []
        for r in self.responses:
            texts.append(r.content) if r.content else None
            texts.extend(tool.content for tool in r.tool_calls) if r.tool_calls else None
        return "\n".join(texts)


ChatResponseStream = Union[
    ChatContentStreamChunk,
    ChatToolCallStreamChunk,
    QuotaResponse,
    ChatFinishMetadataResponse,
    UnknownMetadataResponse,
]

IndexableStreamChunk = Union[
    ChatContentStreamChunk, ChatToolCallStreamChunk, ChatFinishMetadataResponse
]



from typing import Optional, Union

import attrs

from grazie.api.client._common import StopReason, UnknownMetadataResponse
from grazie.api.client.quota.response import QuotaResponse
from grazie.api.client.v8.completion.prompt import CompletionPrompt


@attrs.define(auto_attribs=True, frozen=True)
class CompletionContentStreamChunk:
    content: str


@attrs.define(auto_attribs=True, frozen=True)
class CompletionFinishMetadataResponse:
    reason: StopReason = StopReason.UNKNOWN
    content: str = ""


@attrs.define(auto_attribs=True, frozen=True)
class CompletionResponse:
    prompt: CompletionPrompt
    content: str
    quota: QuotaResponse
    stop_reason: Optional[str] = None
    unknown_metadata: Optional[UnknownMetadataResponse] = None


CompletionResponseStream = Union[
    CompletionContentStreamChunk,
    CompletionFinishMetadataResponse,
    QuotaResponse,
    UnknownMetadataResponse,
]



from typing import Optional

import attrs

from grazie.api.client.v8.chat.roles import ChatRole


@attrs.define(auto_attribs=True, frozen=True)
class ChatMessage:
    def to_json(self) -> dict:
        raise NotImplementedError


@attrs.define(auto_attribs=True, frozen=True)
class TextChatMessage(ChatMessage):
    """
    A message that is just text.
    """

    role: ChatRole
    text: str

    def to_json(self) -> dict:
        return {"type": self.role.value.lower() + "_message", "content": self.text}


@attrs.define(auto_attribs=True, frozen=True)
class AssistantChatMessage(ChatMessage):
    """
    Assistant LLM chat message.
    Used for model-generated messages.

    :param text - for common model-generated messages
    """

    text: Optional[str]

    def to_json(self) -> dict:
        return {
            "type": ChatRole.ASSISTANT.value.lower() + "_message",
            "content": self.text,
        }


@attrs.define(auto_attribs=True, frozen=True)
class AssistantChatMessageTool(ChatMessage):
    id: str
    tool_name: str
    content: str

    def to_json(self) -> dict:
        return {
            "type": ChatRole.ASSISTANT.value.lower() + "_message_tool",
            "id": self.id,
            "toolName": self.tool_name,
            "content": self.content,
        }


@attrs.define(auto_attribs=True, frozen=True)
class ChatToolMessage(ChatMessage):
    id: str
    tool_name: str
    result: str

    def to_json(self) -> dict:
        return {
            "type": ChatRole.TOOL.value.lower() + "_message",
            "id": self.id,
            "toolName": self.tool_name,
            "result": self.result,
        }
